/**
  *  \file planetscentral/host/cmdcron.cc
  *  \brief Host: CRON commands
  */

#include <memory>
#include "planetscentral/host/cmdcron.h"
#include "planetscentral/host/db.h"
#include "planetscentral/dbclient/reconnect.h"
#include "planetscentral/host/cron.h"
#include "planetscentral/host/host.h"
#include "int/array.h"
#include "int/args.h"
#include "planetscentral/host/schedule.h"

static IntValue*
describeJob(const Cron::ScheduleItem& item)
{
    // Result will be hash of action, game, time
    Ptr<IntArrayData> a = new IntArrayData();
    a->content.pushNew(makeStringValue("action"));
    a->content.pushNew(makeStringValue(getCronActionName(item.action)));
    a->content.pushNew(makeStringValue("game"));
    a->content.pushNew(makeIntValue(item.gameId));
    a->content.pushNew(makeStringValue("time"));
    a->content.pushNew(makeIntValue(getUserTimeFromTime(item.time)));
    a->addDimension(a->content.size());
    return new IntArray(a);
}

void
doCronGet(HostConnection& conn, IntArgBlock& args)
{
    // CRONGET gameid
    args.checkArgumentCount(1);
    int32_t gameId = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    Game game(DbSubtree(database_connection, GAME_ROOT), gameId);
    conn.checkPermission(game, Game::ReadPermission);

    std::auto_ptr<IntValue> p(describeJob(cron.getNextAction(gameId)));
    conn.sendObject(p.get());
}

void
doCronList(HostConnection& conn, IntArgBlock& args)
{
    // CRONLIST [LIMIT n]
    bool haveLimit = false;
    int32_t limit = 0;
    while (args.getNumArgs() > 0) {
        string_t keyword = strUCase(toStringResult(args.getNext()));
        if (keyword == "LIMIT") {
            args.checkArgumentCountAtLeast(1);
            limit = toIntResult(args.getNext());
            haveLimit = true;
        } else {
            conn.sendError("400 Syntax error");
            return;
        }
    }

    // Fetch schedules
    std::list<Cron::ScheduleItem> sched;
    cron.getAllSchedules(sched);

    // Generate output. Filter according to logged-in user if required
    ReconnectingGuard g(database_rc);
    Ptr<IntArrayData> a = new IntArrayData();
    for (std::list<Cron::ScheduleItem>::iterator i = sched.begin(); i != sched.end(); ++i) {
        // Limit?
        if (haveLimit && limit == 0) {
            break;
        }

        // Permission check
        if (conn.isAdmin() || Game(DbSubtree(database_connection, GAME_ROOT), i->gameId, Game::NoExistanceCheck).hasPermission(conn.getUser(), Game::ReadPermission)) {
            a->content.pushNew(describeJob(*i));
            if (haveLimit) {
                --limit;
            }
        }
    }
    a->addDimension(a->content.size());

    IntArray result(a);
    conn.sendObject(&result);
}

void
doCronKick(HostConnection& conn, IntArgBlock& args)
{
    // CRONKICK gid
    args.checkArgumentCount(1);
    int32_t gameId = toIntResult(args.getNext());

    // Permission checks. Game must exist (construction of the Game object ensures that),
    // and user must be admin. Mere mortals cannot kick games.
    ReconnectingGuard g(database_rc);
    Game game(DbSubtree(database_connection, GAME_ROOT), gameId);
    conn.checkAdmin();

    // Do it
    bool did = DbSubtree(database_connection, GAME_ROOT).intSetKey("broken").remove(gameId);
    if (did) {
        cron.handleGameChange(gameId);
    }

    IntIntIntValue result(did);
    conn.sendObject(&result);
}
